def rlogList():
    string ='''
        rlogCarga()
        rlogHoldout()
        rlogFmin()
        #BICLASE
        rlogPredict()
        #MULTICLASE
        rlogY_change()
        rlogTraining()
        rlogPredictMulticlass()
        rlogAccuracy()
        rlogMain()
        rlogFull()

    '''
    print(string)

def rlogCarga():
    string = '''
        data = sio.loadmat("ex4data1.mat") # carga de archivo matlab, no es con pd.read_csv()
        X = pd.DataFrame(data['X'])
        y = pd.DataFrame(data['y'])
    '''
    print(string)


def rlogHoldout():
    string = '''
        def holdout(X, y, porcentage=0.7):
            X_training = X.sample(round(percentage*len(X)))  # Selecciona aleatoriamente el numero de filas indicado
            y_training = y.iloc[X_training.index]            # Selecciona las filas del X_training
            X_test = X.iloc[~X.index.isin(X_training.index)] # ~ significa NOT
            y_test = y.iloc[~X.index.isin(X_training.index)] # ~ significa NOT

            print("El tamaño del training debe ser: ", round(percentage*len(X)), " - Comprobación: tamaño X_training es ", len(X_training), " y tamaño y_training es", len(y_training))
            print("El tamaño del test debe ser: ", len(X)-round(percentage*len(X)), " - Comprobación: tamaño X_test es ", len(X_test), " y tamaño y_test es", len(y_test))

            # Reseteamos los índices de todos los conjuntos
            X_training = X_training.reset_index(drop=True)
            y_training = y_training.reset_index(drop=True)
            X_test = X_test.reset_index(drop=True)
            y_test = y_test.reset_index(drop=True)
            
            return X_training, y_training, X_test, y_test

        X_train, y_train, X_test, y_test = holdout(X, y, 0.75)

        '''
    print(string)

def rlogFmin():
    string = '''
        # FUNCIONES PARA FMIN

        def sigmoid(z):
            res = 1/(1+np.exp(-z))
            return res

        def computeCost(thetas, X, y):
            tam = X.shape[0]
            h = sigmoid(np.dot(X, thetas))
            J = 0.0
            J = (-1/tam)*(np.dot(y.T, np.log(h)) + np.dot((1-y).T, np.log(1-h)))
            return J

        def gradient(thetas, X, y):
            tam = X.shape[0]
            h = sigmoid(np.dot(X, thetas))
            J = 0.0
            J = (1/tam)*np.dot(X.T, (h-y))
            return J

        #para ejercicios NO MULTICLASE 
        theta = np.zeros((n+1,1))
        parametros_fmin = op.fmin_cg(maxiter = 200, f=costFunction, x0=theta.flatten(), fprime=gradientFunction, args=(X, y.to_numpy().flatten()))
        print("Thetas encontrado por el fmin_cg: ", parametros_fmin)

    '''
    print(string)

def rlogPredict():
    string= '''
        def predict(theta, X_train, bool_round):
            m = X_train.shape[0] #cuantas filas tendra la prediccion
            p = np.zeros((m,1))
            if bool_round:
                #redondeo
                p = np.round(sigmoid(np.dot(X_train, theta)))
            else:
                #noredondeo
                p = sigmoid(np.dot(X_train, theta))

            return p

    '''
    print(string)

def rlogY_change():
    string = '''
        #FUNCIONES MULTICLASE
        def y_change(y, cl):
            # Convertimos a numpy si y es un DataFrame
            
            if isinstance(y, pd.DataFrame):
                y = y.to_numpy()
            
            y_new = (y == cl).astype(int)  #vector binario (1 donde y == cl, 0 en otro caso)
            y_new = pd.DataFrame({'label': y_new.flatten()}) #flatten pq numpy array es bidimensional, para devolverlo como una lista normal
            return y_new.to_numpy()


    '''
    print(string)

def rlogTraining():
    string = '''
        #FUNCIONES MULTICLASE
        def training(thetas, X_train, y_train, num_clases):
            X_train = X_train.to_numpy()
            y_train = y_train.to_numpy()
            listaThetas = []
            listaCostes = []
            listaClases = []


            for clase in range(1, num_clases+1):
                y_new = y_change(y_train, clase).flatten()
                fmin_resultados = op.fmin_cg(maxiter=50, f=computeCost, x0=thetas.flatten(), fprime=gradient, args=(X_train, y_new.flatten()), full_output = True)
                listaThetas.append(fmin_resultados[0])
                listaCostes.append(fmin_resultados[1])
                listaClases.append(clase)
                
            res_optimization = pd.DataFrame({"thetas": listaThetas, "costes": listaCostes, "clases": listaClases})
            return res_optimization

    '''
    print(string)


def rlogPredictMulticlass():
    string = '''
        def predict(res_optimization, X_test):
            tam = X_test.shape[0] #iteraremos las filas
            indices_filas = [] # lista de los indices de las filas del dataset
            predicciones = [] # prediccion, valor de y para ese conjunto de datos (fila con indice)

            for fila in range(0, tam): #iteramos filas
                lista_h = [] #almacenar las h (probabilidad) de que esa fila pertenezca a cada clase

                for clase in res_optimization["clases"]: #iteramos clases
                    thetas = res_optimization[res_optimization["clases"]==clase]["thetas"].values[0]
                
                    h= sigmoid(np.dot(X_test.iloc[fila], thetas)) #.iloc porque es un dataframe, si fuera numpy ps pondriamos [] y listo.
                    lista_h.append(h)

                prediccion = np.argmax(lista_h)+1 #devuelve el indice de la h mas alta (+1 para que sea la clase y no cuente el 0)
                predicciones.append(prediccion)
                indices_filas.append(fila)

            return pd.DataFrame({"indices_filas": indices_filas, "predicciones": predicciones})

    '''
    print(string)

def rlogAccuracy():
    string = '''
        def accuracy(y_test, y_predicted):
            return np.mean(y_predicted["predicciones"]==y_test[0])

    '''
    print(string)

def rlogMain():
    string = '''
        # Paso 1:
        num_classes = 10
        # Columna de 1s en la primera posición de X está ya hecho, comprobar
        display(X)

        # si no esta hecho es así:
        ones = np.ones((len(X), 1))
        X.insert(0, "uno", ones)
        #print(X)

        # Paso 2: HOLDOUT 70-30
        X_train, X_test, y_train, y_test = holdout(X, y, 0.7)
        thetas = np.zeros((X_train.shape[1],1))

        # Paso 3: ENTRENAMIENTO
        res_optimization_training = training(thetas, X_train, y_train, num_classes)

        # Paso 4: PREDICCION Y ACCURACY
        res_prediction_training = predict(res_optimization_training, X_test) # PREDICCIÓN DEL TRAINING
        accuracy_training = accuracy(y_test, res_prediction_training) # ACCURACY DE LA PREDICCIÓN DEL TRAINING
        print("Training accuracy: ", accuracy_training)
        #print("Training accuracy sklearn:", metrics.accuracy_score(y_train,res_prediction_training['predicciones']))

    '''
    print(string)


def rlogFull():
    rlogCarga()
    print("\n\n")
    rlogHoldout()
    print("\n\n")
    rlogFmin()
    print("\n\n")
    #BICLASE
    print("\n\n")
    rlogPredict()
    #MULTICLASE
    print("\n\n")
    rlogY_change()
    print("\n\n")
    rlogTraining()
    print("\n\n")
    rlogPredictMulticlass()
    print("\n\n")
    rlogAccuracy()
    print("\n\n")
    rlogMain()
    






def utilPlot():
    string = '''
        import matplotlib.pyplot as plt
        from matplotlib import cm
        import numpy as np
        # plotData plots the data points x and y into a new figure
        # plotData(x,y) plots the data points and gives the figure axes labels of population and profit.
        from computeCost import computeCost

        ################################
        ######### EJERCICIO 01 #########
        ################################
        def plotData(X, y):
            # ====================== YOUR CODE HERE ======================
            # Instructions: Plot the training data into a figure using the "scatter" and "show"
            #               commands of the library "matplotlib".
            #               Set the axes labels using the "xlabel" and "ylabel" commands.
            #               Assume the population and revenue data have been passed in as the X and y arguments of this function.
            # Hint: You can use the 'x' and 'red' options with plot to have the markers appear as red crosses.

            
        ################################
        ################################

        def plotIterationsVsCost(J_history, alpha):
            plt.plot(J_history['iteracion'], J_history['cost'])
            plt.xlabel('Iterations')
            plt.ylabel('Cost')
            plt.title('Plot to check if gradient descent is working correctly with alpha: '+str(alpha))
            plt.show()

        def plotDataLogisticRegression(data):
            admitidos = data[data["label"]==1]
            noadmitidos = data[data["label"]==0]
            #admitidos =data.iloc[data["label"==1]]

            plt.scatter(admitidos["score1"], admitidos["score2"], label="Admitidos", c="blue", marker="+")
            plt.scatter(noadmitidos["score1"], noadmitidos["score2"], label="No admitidos", c="yellow", marker=".")

            plt.xlabel("Puntuacion examen 1")
            plt.ylabel("Puntuacion examen 2")
            plt.legend()
            plt.show()


        def plotData_linearRegression(X, y, theta):
            plt.scatter(x=X['poblacion'], y=y, marker="x", c="red", label="Training data")
            plt.plot(X['poblacion'], np.dot(X, theta), c='blue', label="Linear regression") # h  -> mi modelo
            plt.xlabel("Population of City in 10,000")
            plt.ylabel("Profit in $10,000")
            plt.xticks(np.arange(5, 26, 5))
            plt.xlim(5, 25)
            plt.ylim(-5, 25)
            plt.legend()
            plt.show()

        def plotData_cost(X, y, theta):
            # Grid over which we will calculate J
            theta0_vals = np.linspace(-10, 10, 100)
            theta1_vals = np.linspace(-1, 4, 100)

            # Initialize J_vals to a matrix of 0's
            J_vals = np.zeros((len(theta0_vals), len(theta1_vals)))

            # Fill out J_vals
            for i in range(1, len(theta0_vals)):
                for j in range(1, len(theta1_vals)):
                    theta_ij = [[theta0_vals[i]], [theta1_vals[j]]]
                    J_vals[i][j] = computeCost(X, y, theta_ij)

            # Because of the way meshgrids work in the surf command, we need to
            # transpose J_vals before calling surf, or else the axes will be flipped
            J_vals = J_vals.T

            # Surface plot
            fig = plt.figure()
            ax = fig.add_subplot(projection='3d')
            theta0_vals, theta1_vals = np.meshgrid(theta0_vals, theta1_vals)
            surf = ax.plot_surface(theta0_vals, theta1_vals, J_vals, cmap = cm.jet, rstride = 2, cstride = 2) #linewidth = 0, antialiased = False)
            #fig.colorbar(surf, shrink = 0.5, aspect = 5)
            plt.xlabel("theta_0")
            plt.ylabel("theta_1")
            plt.show()

            fig1, bx = plt.subplots(1,1)
            contour = bx.contour(theta0_vals, theta1_vals, J_vals, np.logspace(-2, 3, 20))
            fig1.colorbar(contour)
            plt.xlabel("theta_0")
            plt.ylabel("theta_1")
            plt.plot(theta[0], theta[1], marker = 'x', c='red')
            plt.show()

        def plotData_problema3(X, y, theta):
            plt.scatter(X[['tam']], y[['precio']], marker="x", c="red", label="Training data")
            plt.plot(X[['tam']], np.dot(X[['uno','tam']], theta), c='blue', label="Linear regression")
            plt.xlabel("Population of City in 10,000")
            plt.ylabel("Profit in $10,000")
            plt.xlim(5, 25)
            plt.ylim(-5, 25)
            plt.legend()
            plt.show()
    '''
    print(string)

def check():
    string = '''
        def checkNNGradients(lambda_param):
        input_layer_size = 3
        hidden_layer_size_c1 = 5
        hidden_layer_size_c2 = 4
        num_labels = 3
        m = 5

        Theta1 = debugInitializeWeights(hidden_layer_size_c1, input_layer_size)
        Theta2 = debugInitializeWeights(hidden_layer_size_c2, hidden_layer_size_c1)
        Theta3 = debugInitializeWeights(num_labels, hidden_layer_size_c2)

        X = debugInitializeWeights(m,input_layer_size-1)
        y = np.zeros(m)
        for i in range(m):
            y[i] = (1 + mt.fmod(i+1,num_labels))

        nn_params = np.hstack((Theta1.ravel(order='F'), Theta2.ravel(order='F'), Theta3.ravel(order='F')))

        nn_backprop_params = nnGradFunction(nn_params, input_layer_size, hidden_layer_size_c1, hidden_layer_size_c2, num_labels, X, y)

        mygrad = computeNumericalGradient(nn_params, input_layer_size, hidden_layer_size_c1, hidden_layer_size_c2, num_labels,X, y)

        df = pd.DataFrame(mygrad,nn_backprop_params)
        print(df)

        diff = np.linalg.norm((mygrad-nn_backprop_params))/np.linalg.norm((mygrad+nn_backprop_params))
        print('If your backpropagation implementation is correct, then the differences will be small (less than 1e-9):' , diff)

    def debugInitializeWeights(fan_out, fan_in):
        W = np.zeros((fan_out,1+fan_in))
        b = np.zeros(W.size)
        for i in np.array(range(1,W.size+1)):
            b[i-1] = mt.sin(i)
        W = np.reshape(b,W.shape,order='F') / 10
        return W

    def computeNumericalGradient(theta, input_layer_size, hidden_layer_size_c1, hidden_layer_size_c2, num_labels,X, y):
        mygrad = np.zeros(theta.size)
        perturb = np.zeros(theta.size)
        myeps = 0.0001
        for i in range(np.size(theta)):
            perturb[i] = myeps
            cost_high = nnCostFunction(theta + perturb, input_layer_size,
                                            hidden_layer_size_c1, hidden_layer_size_c2, num_labels,X, y)
            cost_low = nnCostFunction(theta - perturb, input_layer_size,
                                            hidden_layer_size_c1, hidden_layer_size_c2, num_labels,X, y)
            mygrad[i] = (cost_high - cost_low) / float(2 * myeps)
            perturb[i] = 0
        return mygrad
    '''
    print(string)


def rlinList():
    string = '''
        rlinCarga()
        rlinPlotData()
        rlinComputeCost()
        rlinGradFunction()
        rlinPlotCostIter()
        rlinHoldout()
        rlinError()
        rlinFull()
        rlinNormalize()
    '''
    print(string)

def rlinCarga():
    string = '''
        file_name = 'ex1data1.txt'
        print('Loading Data ...', file_name)
        file = pd.read_csv(file_name, names=["poblacion","beneficio"])
        X = pd.DataFrame({'poblacion': file['poblacion']})
        y = pd.DataFrame({'beneficio': file['beneficio']})
    '''
    print(string)

def rlinPlotData():
    string = '''
        def plotData(X, y):
            plt.scatter(X,y, marker = "x", c = "red")
            plt.xlabel("Population of City in 10,000")
            plt.ylabel("Profit in $10,000")
            plt.xticks(np.arange(5, 26, 5), rotation=45) 
            plt.yticks(np.arange(-5, 26, 5), rotation=45)
            plt.xlim(5,25) # Estamos limitando el eje x. Si los datos cambian, probablemente habrá que modificar estos valores
            plt.ylim(-5,25) # Estamos limitando el eje y. Si los datos cambian, probablemente habrá que modificar estos valores
            plt.show()

        print("El tamaño de X es: ", X.shape[0], " filas y ", X.shape[1], " columna(s) ", X.shape)
        print("La longitud del vector y es: ", len(y), y.shape)

        print("Las 5 primeras filas de los datos son: [atributo entrada X1 | clase y]")
        for i in range(0,5):
        print("\t", X['poblacion'][i],  " | ", y['beneficio'][i]) 
        # Recordar: X[:5] selecciona las 5 primeras filas de X.

        plotData(X, y)
    '''
    print(string)

def rlinNormalize():
    string = '''
        def normalize(X):
            mu = X.mean()
            sigma = X.std()
            X_norm = (X-mu)/(sigma)
            
            return X_norm, mu, sigma

        X, mu, sigma = featureNormalize(X)
        display(X.head(10))
        print("MEDIA:", mu, "\nSTD:", sigma)

    '''
    print(string)

def rlinComputeCost():
    string = '''
        def computeCost(X, y, theta):
            m = len(y) # Numero de instancias en el training
            h = np.dot(X, theta) # Hipótesis del modelo de regresión lineal 
            J = np.sum(np.power((h - y),2), axis=0)/(2*m) # Coste
            #J = (1/(2*m))*np.dot((h-y).T,(h-y))
            return J

        ones = np.ones((len(y), 1)) # Crear un array de 1
        X['uno'] = ones # Añadir a X # Es lo mismo que X['uno'] = 1
        X = X[['uno', 'poblacion']]  # Poner 'uno' como primera columna

        num_atributos = X.shape[1] # Si esta operación la hacemos antes de añadir la columna de 1 a X, debemos poner X.shape[1]+1
        theta = np.zeros((num_atributos,1), dtype=np.float64)

        J_base = computeCost(X, y, theta)
        print("\nEl coste inicializando theta a 0 debe ser aproximadamente 32.072734: ", J_base)

        # Si queremos probar con otros valores de theta: 
        J_prueba = computeCost(X, y, [[-1], [2]]) # Lo mismo que: computeCost(X, y, np.array([[-1],[2]]))
        print("\nEl coste probando con theta0=-1 y theta1=2 es: ", J_prueba)
    '''
    print(string)

def rlinGradFunction():
    string = '''
        def gradientDescent(X, y, theta, alpha, iterations):
            m = len(y) # Numero de instancias en el training
            current_iter = [] # Lista vacía para crear el histórico en un dataframe
            current_cost = [] # Lista vacía para crear el histórico en un dataframe

            for iter in range(iterations):
                h = np.dot(X,theta) # Hipótesis
                theta = theta - alpha*(1/m)*(np.dot(X.T,(h-y)))

                # Guardar el coste J de cada iteración
                current_iter.append(iter) # Añadir la iteración a una lista
                current_cost.append(computeCost(X, y, theta)) # Añadir el coste a una lista

            J_history = pd.DataFrame({'iteracion': current_iter, 'coste': current_cost}) # Crear el dataframe histórico iteracion-coste

            return theta, J_history # Último theta encontrado y dataframe histórico J_history
    
        
        iterations = 1500
        alpha = 0.01
        theta_optimo_holdout, J_history_holdout = gradientDescent(X_training, y_training, theta, alpha, iterations)

        print('Theta encontrado con el descenso del gradiente: \n', theta_optimo_holdout)
        print('\nCoste alcanzado en la última iteración : ', J_history_holdout[J_history_holdout['iteracion']==iterations-1]['coste'])
    '''
    print(string)

def rlinPlotCostIter():
    string = '''
        def plotIterationsVsCost(J_history, alpha, iteraciones):
            plt.plot(J_history['iteracion'], J_history['coste'])
            plt.xlabel('Iteraciones')
            plt.ylabel('Coste')
            plt.title('Descenso del gradiente con alpha: '+str(alpha)+' y '+str(iteraciones)+' iteraciones')
            plt.show()

        def plotData_cost(X, y, theta):

            # Grid sobre el que vamos a calcular J
            theta0_vals = np.linspace(-10, 10, 100)
            theta1_vals = np.linspace(-1, 4, 100)

            # Inicializar J_vals a una matriz de ceros
            J_vals = np.zeros((len(theta0_vals), len(theta1_vals)))

            # Rellenar J_vals
            for i in range(1, len(theta0_vals)):
                for j in range(1, len(theta1_vals)):
                    theta_ij = [[theta0_vals[i]], [theta1_vals[j]]]
                    J_vals[i][j] = computeCost(X, y, theta_ij)

            # Debido a la forma en la que las mallas funcionan en el comando contour, 
            # debemos transponer J_vals antes de llamar a dicha función
            J_vals = J_vals.T

            fig1, ax = plt.subplots(1,1)
            contour = ax.contour(theta0_vals, theta1_vals, J_vals, np.logspace(-2, 3, 20))
            fig1.colorbar(contour)
            plt.xlabel("theta_0")
            plt.ylabel("theta_1")
            plt.plot(theta[0], theta[1], marker = 'x', c='red')
            plt.show()

        def plotData_linearRegression(X, y, theta):
            plt.scatter(X['poblacion'],y, marker="x", c="red", label="Training data") # Representación del conjunto de datos
            plt.plot(X['poblacion'], np.dot(X, theta), c='blue', label="Linear regression") # Representación de la recta: h = X·theta
            plt.xlabel("Population of City in 10,000")
            plt.ylabel("Profit in $10,000")
            plt.xlim(5, 25) # Cuidado: estamos limitando el eje X 
            plt.ylim(-5, 25) # Cuidado: estamos limitando el eje y
            plt.legend()
            plt.show()
    '''
    print(string)

def rlinHoldout():
    string = '''
        def holdout(X, y, percentage=0.6):
            X_training = X.sample(round(percentage*len(X))) # Selecciona aleatoriamente el numero de filas indicado
            y_training = y.iloc[X_training.index] # Selecciona las filas del X_training
            X_test = X.iloc[~X.index.isin(X_training.index)] # ~ significa NOT
            y_test = y.iloc[~X.index.isin(X_training.index)] # ~ significa NOT

            print("El tamaño del training debe ser: ", round(percentage*len(X)), " - Comprobación: tamaño X_training es ", len(X_training), " y tamaño y_training es", len(y_training))
            print("El tamaño del test debe ser: ", len(X)-round(percentage*len(X)), " - Comprobación: tamaño X_test es ", len(X_test), " y tamaño y_test es", len(y_test))

            # Reseteamos los índices de todos los conjuntos
            X_training = X_training.reset_index(drop=True)
            y_training = y_training.reset_index(drop=True)
            X_test = X_test.reset_index(drop=True)
            y_test = y_test.reset_index(drop=True)
            
            return X_training, y_training, X_test, y_test

            # OPCION 3
        def holdout(X, y, percentage=0.6):
            X_training, X_test, y_training, y_test = train_test_split(X,y, test_size=1-percentage, random_state=1) # Cuidado con el orden de la salida
            # random_state: controla el barajado aplicado a los datos antes de aplicar la división. Si le pasamos un int, la salida se reproducirá siempre que llamemos a la función
            
            print("El tamaño del training debe ser: ", round(percentage*len(X)), " - Comprobación: tamaño X_training es ", len(X_training), " y tamaño y_training es", len(y_training))
            print("El tamaño del test debe ser: ", len(X)-round(percentage*len(X)), " - Comprobación: tamaño X_test es ", len(X_test), " y tamaño y_test es", len(y_test))
            
            # Reseteamos los índices de todos los conjuntos
            X_training = X_training.reset_index(drop=True)
            y_training = y_training.reset_index(drop=True)
            X_test = X_test.reset_index(drop=True)
            y_test = y_test.reset_index(drop=True)

            return X_training, y_training, X_test, y_test

        X_training, y_training, X_test, y_test = holdout(X, y, 0.7)
    '''
    print(string)

def rlinError():
    string = '''
        y_predicted = np.dot(X_test, theta_optimo_holdout)

        m = len(y_test)

        # MAE: 
        error_absoluto_medio_holdout = (1/m) * np.sum((np.abs(y_test-y_predicted)), axis=0)
        print("El error absoluto medio es: ", error_absoluto_medio_holdout)
        MAE_sklearn_holdout = metrics.mean_absolute_error(y_test, y_predicted)
        print("El error absoluto medio usando sklearn es: ", MAE_sklearn_holdout)

        # MSE: 
        error_cuadratico_medio_holdout = (1/m) * np.sum(np.power((y_test-y_predicted),2), axis=0)
        print("\n\nEl error cuadrático medio es: ", error_cuadratico_medio_holdout)
        MSE_sklearn_holdout = metrics.mean_squared_error(y_test, y_predicted)
        print("El error cuadrático medio usando sklearn es: ", MSE_sklearn_holdout)
    '''
    print(string)

def rlinFull():
    rlinCarga()
    print("\n\n")
    rlinPlotData()
    print("\n\n")
    rlinComputeCost()
    print("\n\n")
    rlinGradFunction()
    print("\n\n")
    rlinPlotCostIter()
    print("\n\n")
    rlinHoldout()
    print("\n\n")
    rlinError()
    print("\n\n")
    rlinNormalize()


def rnList():
    string = '''
        rnCarga()
        rnHoldout()
        rnNormalize()
        rnRandInitializeWeights()
        rnForward()
        rnCostFunction()
        rnGradFunction()
        rnTraining()
        rnPredict()
        rnFull()

    '''
    print(string)

def rnCarga():
    string = '''
        CargaOpcion1:
            import pandas as pd
            data = pd.read_csv("drivers_behavior.csv")
            y = pd.DataFrame({'target': data['Target']})
            X = data.drop(['Target'], axis=1)
            # Definición parámetros RED NEURONAL
        
        CargaOpcion2:
            import scipy.io as sio
            data = sio.loadmat("ex4data1.mat") # tipo dict
            X = data['X']
            y = data['y']
            m = X.shape[0]
            # Definición parámetros RED NEURONAL
    '''
    print(string)

def rnHoldout():
    string = '''
        def holdout(X, y, porcentage=0.7):
            X_training = X.sample(round(percentage*len(X)))  # Selecciona aleatoriamente el numero de filas indicado
            y_training = y.iloc[X_training.index]            # Selecciona las filas del X_training
            X_test = X.iloc[~X.index.isin(X_training.index)] # ~ significa NOT
            y_test = y.iloc[~X.index.isin(X_training.index)] # ~ significa NOT

            print("El tamaño del training debe ser: ", round(percentage*len(X)), " - Comprobación: tamaño X_training es ", len(X_training), " y tamaño y_training es", len(y_training))
            print("El tamaño del test debe ser: ", len(X)-round(percentage*len(X)), " - Comprobación: tamaño X_test es ", len(X_test), " y tamaño y_test es", len(y_test))

            # Reseteamos los índices de todos los conjuntos
            X_training = X_training.reset_index(drop=True)
            y_training = y_training.reset_index(drop=True)
            X_test = X_test.reset_index(drop=True)
            y_test = y_test.reset_index(drop=True)
            
            return X_training, y_training, X_test, y_test

        X_train, y_train, X_test, y_test = holdout(X, y, 0.75)
    '''
    print(string)


def rnNormalize():
    string = '''
        def normalize(X):
            mu = X.mean()
            sigma = X.std()
            X_norm = (X-mu)/(sigma)
            
            return X_norm, mu, sigma

        X_train, mu_train, sigma_train = normalize(X_train)
        X_test, mu_test, sigma_test = normalize(X_test)
    
    '''
    print(string)

def rnRandInitializeWeights():
    string = '''
        def randInitializeWeights(capa_entrada, capa_salida):
            epsilon_init = 0.12
            W = np.random.rand(capa_salida, capa_entrada) * 2 * epsilon_init - epsilon_init
            return W
    
    '''
    print(string)

def rnForward():
    string = '''
        def sigmoid(z):
            res = 1/(1+np.exp(-z))
            return res

        def forward(X, theta1, theta2, theta3):
            #variables
            m = len(X)
            ones = np.ones((m, 1))
            
            #activaciones
            a1 = np.hstack((ones, X))
            a2 = sigmoid(np.dot(a1, theta1.T))
            
            a2 = np.hstack((ones, a2))
            a3 = sigmoid(np.dot(a2, theta2.T))
            
            a3 = np.hstack((ones, a3))
            a4 = sigmoid(np.dot(a3, theta3.T))
            
            return a1, a2, a3, a4
    '''
    print(string)

def rnCostFunction():
    string = '''
        def nnCostFunction(nn_params, input_layer_size, hidden_layer_size1, hidden_layer_size2, num_labels, X, y):
            #variables
            m = len(X)
            
            #desenrollar thetas
            inicio = 0
            fin = hidden_layer_size1 * (input_layer_size+1)
            theta1 = np.reshape(nn_params[:fin], (hidden_layer_size1, input_layer_size + 1), 'F')
            
            inicio = fin
            fin = fin + hidden_layer_size2 * (hidden_layer_size1+1)
            theta2 = np.reshape(nn_params[inicio:fin], (hidden_layer_size2, hidden_layer_size1 + 1), 'F')
            
            theta3 = np.reshape(nn_params[fin:], (num_labels, hidden_layer_size2+1), 'F')
            
            #calculo activaciones
            a1, a2, a3, h = forward(X, theta1, theta2, theta3)
            
            #getdummies
            y_d = pd.get_dummies(y.flatten())
            
            #calculo coste
            #J = (-1/m) * np.sum(np.sum(np.multiply(y_d,np.log(h)) + np.multiply((1- y_d), np.log(1-h))))
            J = (-1/m)*np.sum(np.dot(y_d.T, np.log(h))+ np.dot((1-y_d).T, np.log(1-h)))

            return J
    '''
    print(string)

def rnGradFunction():
    string= '''
        def nnGradFunction(nn_params, input_layer_size, hidden_layer_size1, hidden_layer_size2, num_labels, X, y):
            #variables
            m = len(X)
            
            #desenrollar thetas
            inicio = 0
            fin = hidden_layer_size1 * (input_layer_size+1)
            theta1 = np.reshape(nn_params[inicio:fin], (hidden_layer_size1, input_layer_size + 1), 'F')
            
            inicio = fin
            fin = fin + hidden_layer_size2 * (hidden_layer_size1+1)
            theta2 = np.reshape(nn_params[inicio:fin], (hidden_layer_size2, hidden_layer_size1 + 1), 'F')
            
            theta3 = np.reshape(nn_params[fin:], (num_labels, hidden_layer_size2+1), 'F')
            
            #calculo activaciones
            a1, a2, a3, a4 = forward(X, theta1, theta2, theta3)
            
            #getdummies
            y_d = pd.get_dummies(y.flatten())
            
            #calculo deltas
                #inicializacion
            delta1 = np.zeros(theta1.shape)
            delta2 = np.zeros(theta2.shape)
            delta3 = np.zeros(theta3.shape)
            
                #d's
            d4 = a4 - y_d
            d3 = np.multiply(np.dot(d4, theta3), np.multiply(a3, (1-a3)))
            d2 = np.multiply(np.dot(d3[:,1:], theta2), np.multiply(a2,(1-a2)))
            
            d3 = d3[:,1:]
            d2 = d2[:,1:]
            
                #producto deltas
            delta1 = d2.T @ a1
            delta2 = d3.T @ a2
            delta3 = d4.T @ a3
            
                #normalizacion
            delta1 /= m
            delta2 /= m
            delta3 /= m
            
            delta3 = delta3.to_numpy()
            
            #unroll
            gradiente = np.hstack((delta1.ravel(order='F'), delta2.ravel(order='F'), delta3.ravel(order='F')))
            return gradiente
    '''
    print(string)

def rnTraining():
    string= '''
        theta1 = randInitializeWeights(input_layer_size +1, hidden_layer_size1)
        theta2 = randInitializeWeights(hidden_layer_size1 +1, hidden_layer_size2)
        theta3 = randInitializeWeights(hidden_layer_size2 +1, num_labels)

        nn_initial_params = np.hstack((theta1.ravel(order='F'), theta2.ravel(order='F'), theta3.ravel(order='F')))

        maxiter = 10
        X_train = X_train.to_numpy()
        y_train = y_train.to_numpy()
        print(X_train.shape)
        print(y_train.shape)

        nn_params = opt.fmin_cg(maxiter=maxiter, f=nnCostFunction, x0=nn_initial_params, fprime=nnGradFunction, args=(input_layer_size, hidden_layer_size1, hidden_layer_size2, num_labels, X_train, y_train.flatten()))

        # desenrollar thetas -----

        inicio = 0
        fin = hidden_layer_size1 * (input_layer_size+1)
        theta_opt1 = np.reshape(nn_params[inicio:fin], (hidden_layer_size1, input_layer_size + 1), 'F')

        inicio = fin
        fin = fin + hidden_layer_size2 * (hidden_layer_size1+1)
        theta_opt2 = np.reshape(nn_params[inicio:fin], (hidden_layer_size2, hidden_layer_size1 + 1), 'F')

        theta_opt3 = np.reshape(nn_params[fin:], (num_labels, hidden_layer_size2+1), 'F')
    '''
    print(string)

def rnPredict():
    string = '''
        def predict(theta1, theta2, theta3, X):
            a1, a2, a3, h = forward(X, theta1, theta2, theta3)
            pred = np.argmax(h, axis=1) + 1  
            return pred

        y_test = y_test.to_numpy()
        pred = predict(theta_opt1, theta_opt2, theta_opt3, X_test)
        print("Accuracy: ", np.mean(pred== y_test.flatten())*100)
    '''
    print(string)

def rnFull():
    rnCarga()
    print("\n\n")
    rnHoldout()
    print("\n\n")
    rnNormalize()
    print("\n\n")
    rnRandInitializeWeights()
    print("\n\n")
    rnForward()
    print("\n\n")
    rnCostFunction()
    print("\n\n")
    rnGradFunction()
    print("\n\n")
    rnTraining()
    print("\n\n")
    rnPredict()


from setuptools import setup, find_packages

setup(
    name="sklearmpi",  # Nombre del paquete
    version="0.1.1",  # Versión inicial
    author="Tu Nombre",
    author_email="tu_email@ejemplo.com",
    description="Una descripción breve de tu biblioteca",
    url="https://github.com/tu_usuario/mi_libreria",  # URL del repositorio
    packages=find_packages(),  # Encuentra automáticamente submódulos
)